@@ -1,25 +1,34 @@
 from collections import OrderedDict
+from typing import Type
 
 from django import forms
 from django.conf import settings
 from django.contrib import admin, messages
 from django.contrib.admin.widgets import FilteredSelectMultiple
 from django.contrib.auth import get_user_model
+from django.http import HttpRequest
 from django.shortcuts import get_object_or_404, redirect, render
 from django.urls import reverse, path
-from django.utils.translation import ugettext_lazy as _
-from django.utils.translation import ugettext
+from django.utils.translation import gettext
+from django.utils.translation import gettext_lazy as _
+
 from guardian.forms import GroupObjectPermissionsForm, UserObjectPermissionsForm
-from guardian.models import Group
-from guardian.shortcuts import (get_group_perms, get_groups_with_perms, get_perms_for_model, get_user_perms,
-                                get_users_with_perms)
+from guardian.shortcuts import (
+    get_group_perms,
+    get_groups_with_perms,
+    get_perms_for_model,
+    get_user_perms,
+    get_users_with_perms,
+)
+from guardian.utils import get_group_obj_perms_model
 
 
 class AdminUserObjectPermissionsForm(UserObjectPermissionsForm):
-    """
-    Extends :form:`UserObjectPermissionsForm`. It only overrides
-    ``get_obj_perms_field_widget`` method so it return
-    ``django.contrib.admin.widgets.FilteredSelectMultiple`` widget.
+    """Admin form for user object permissions.
+
+    Extends the `UserObjectPermissionsForm` and overrides the
+    `get_obj_perms_field_widget` method so it returns the
+    `django.contrib.admin.widgets.FilteredSelectMultiple` widget.
     """
 
     def get_obj_perms_field_widget(self):
@@ -27,10 +36,11 @@ class AdminUserObjectPermissionsForm(UserObjectPermissionsForm):
 
 
 class AdminGroupObjectPermissionsForm(GroupObjectPermissionsForm):
-    """
-    Extends :form:`GroupObjectPermissionsForm`. It only overrides
-    ``get_obj_perms_field_widget`` method so it return
-    ``django.contrib.admin.widgets.FilteredSelectMultiple`` widget.
+    """Admin form for group object permissions.
+
+    Extends the `GroupObjectPermissionsForm` and overrides the
+    `get_obj_perms_field_widget` method so it returns the
+    `django.contrib.admin.widgets.FilteredSelectMultiple` widget.
     """
 
     def get_obj_perms_field_widget(self):
@@ -38,22 +48,21 @@ class AdminGroupObjectPermissionsForm(GroupObjectPermissionsForm):
 
 
 class GuardedModelAdminMixin:
-    """
-    Serves as a helper for custom subclassing ``admin.ModelAdmin``.
-    """
-    change_form_template = \
-        'admin/guardian/model/change_form.html'
-    obj_perms_manage_template = \
-        'admin/guardian/model/obj_perms_manage.html'
-    obj_perms_manage_user_template = \
-        'admin/guardian/model/obj_perms_manage_user.html'
-    obj_perms_manage_group_template = \
-        'admin/guardian/model/obj_perms_manage_group.html'
-    user_can_access_owned_objects_only = False
-    user_owned_objects_field = 'user'
-    user_can_access_owned_by_group_objects_only = False
-    group_owned_objects_field = 'group'
-    include_object_permissions_urls = True
+    """Mixin helper for custom subclassing `admin.ModelAdmin`."""
+
+    change_form_template: str = "admin/guardian/model/change_form.html"
+    obj_perms_manage_template: str = "admin/guardian/model/obj_perms_manage.html"
+    obj_perms_manage_user_template: str = (
+        "admin/guardian/model/obj_perms_manage_user.html"
+    )
+    obj_perms_manage_group_template: str = (
+        "admin/guardian/model/obj_perms_manage_group.html"
+    )
+    user_can_access_owned_objects_only: bool = False
+    user_owned_objects_field: str = "user"
+    user_can_access_owned_by_group_objects_only: bool = False
+    group_owned_objects_field: str = "group"
+    include_object_permissions_urls: bool = True
 
     def get_queryset(self, request):
         qs = super().get_queryset(request)
@@ -65,24 +74,21 @@ class GuardedModelAdminMixin:
             filters = {self.user_owned_objects_field: request.user}
             qs = qs.filter(**filters)
         if self.user_can_access_owned_by_group_objects_only:
-            User = get_user_model()
-            user_rel_name = User.groups.field.related_query_name()
-            qs_key = '%s__%s' % (self.group_owned_objects_field, user_rel_name)
-            filters = {qs_key: request.user}
+            qs_key = f'{self.group_owned_objects_field}__in'
+            filters = {qs_key: request.user.groups.all()}
             qs = qs.filter(**filters)
         return qs
 
     def get_urls(self):
         """
+
         Extends standard admin model urls with the following:
+        - `.../permissions/` under `app_mdodel_permissions` url name (params: object_pk)
+        - `.../permissions/user-manage/<user_id>/` under `app_model_permissions_manage_user` url name (params: object_pk, user_pk)
+        - `.../permissions/group-manage/<group_id>/` under `app_model_permissions_manage_group` url name (params: object_pk, group_pk)
 
-        - ``.../permissions/`` under ``app_mdodel_permissions`` url name (params: object_pk)
-        - ``.../permissions/user-manage/<user_id>/`` under ``app_model_permissions_manage_user`` url name (params: object_pk, user_pk)
-        - ``.../permissions/group-manage/<group_id>/`` under ``app_model_permissions_manage_group`` url name (params: object_pk, group_pk)
-
-        .. note::
-           ``...`` above are standard, instance detail url (i.e.
-           ``/admin/flatpages/1/``)
+        Note:
+           `...` above are standard, instance detail url (i.e. `/admin/flatpages/1/`)
 
         """
         urls = super().get_urls()
@@ -106,10 +112,14 @@ class GuardedModelAdminMixin:
         return urls
 
     def get_obj_perms_base_context(self, request, obj):
-        """
+        """Get context dict with common admin and object permissions related content.
+
         Returns context dictionary with common admin and object permissions
         related content. It uses AdminSite.each_context,
         making sure all required template vars are in the context.
+
+        Returns:
+            django template context
         """
         context = self.admin_site.each_context(request)
         context.update({
@@ -126,12 +136,11 @@ class GuardedModelAdminMixin:
         return context
 
     def obj_perms_manage_view(self, request, object_pk):
-        """
-        Main object permissions view. Presents all users and groups with any
-        object permissions for the current model *instance*. Users or groups
-        without object permissions for related *instance* would **not** be
-        shown. In order to add or manage user or group one should use links or
-        forms presented within the page.
+        """Main object permissions view.
+
+        Presents all users and groups with any object permissions for the current model *instance*.
+        Users or groups without object permissions for related *instance* would **not** be shown.
+        To add or manage user or group, one should use links or forms presented within the page.
         """
         if not self.has_change_permission(request, None):
             post_url = reverse('admin:index', current_app=self.admin_site.name)
@@ -210,9 +219,9 @@ class GuardedModelAdminMixin:
         Returns main object permissions admin template.  May be overridden if
         need to change it dynamically.
 
-        .. note::
-           If ``INSTALLED_APPS`` contains ``grappelli`` this function would
-           return ``"admin/guardian/grappelli/obj_perms_manage.html"``.
+        Note:
+           If `INSTALLED_APPS` contains `grappelli` this function would
+           return `"admin/guardian/grappelli/obj_perms_manage.html"`.
 
         """
         if 'grappelli' in settings.INSTALLED_APPS:
@@ -220,9 +229,7 @@ class GuardedModelAdminMixin:
         return self.obj_perms_manage_template
 
     def obj_perms_manage_user_view(self, request, object_pk, user_id):
-        """
-        Manages selected users' permissions for current object.
-        """
+        """Manages selected users' permissions for the current object."""
         if not self.has_change_permission(request, None):
             post_url = reverse('admin:index', current_app=self.admin_site.name)
             return redirect(post_url)
@@ -234,7 +241,7 @@ class GuardedModelAdminMixin:
 
         if request.method == 'POST' and form.is_valid():
             form.save_obj_perms()
-            msg = ugettext("Permissions saved.")
+            msg = gettext("Permissions saved.")
             messages.success(request, msg)
             info = (
                 self.admin_site.name,
@@ -256,57 +263,70 @@ class GuardedModelAdminMixin:
 
         return render(request, self.get_obj_perms_manage_user_template(), context)
 
-    def get_obj_perms_manage_user_template(self):
-        """
-        Returns object permissions for user admin template.  May be overridden
-        if need to change it dynamically.
+    def get_obj_perms_manage_user_template(self) -> str:
+        """Returns object permissions for user admin template.
 
-        .. note::
-           If ``INSTALLED_APPS`` contains ``grappelli`` this function would
-           return ``"admin/guardian/grappelli/obj_perms_manage_user.html"``.
+        May be overridden if dynamic behavior is needed.
 
+        Note:
+           If `INSTALLED_APPS` contains "grappelli" this function returns
+           `"admin/guardian/grappelli/obj_perms_manage_user.html"`.
+           Else, it returns `self.obj_perms_manage_user_template`.
         """
         if 'grappelli' in settings.INSTALLED_APPS:
             return 'admin/guardian/contrib/grappelli/obj_perms_manage_user.html'
         return self.obj_perms_manage_user_template
 
-    def get_obj_perms_user_select_form(self, request):
-        """
-        Returns form class for selecting a user for permissions management.  By
-        default :form:`UserManage` is returned.
+    def get_obj_perms_user_select_form(self, request: HttpRequest) -> Type[forms.Form]:
+        """Get the form class for selecting a user for permissions management.
+
+        Parameters:
+            request (HttpRequest): The HTTP request object.
+
+        Returns:
+            The form class for selecting a user for permissions management.
+                Default is `UserManage`
         """
         return UserManage
 
-    def get_obj_perms_group_select_form(self, request):
-        """
-        Returns form class for selecting a group for permissions management.  By
-        default :form:`GroupManage` is returned.
+    def get_obj_perms_group_select_form(self, request: HttpRequest) -> Type[forms.Form]:
+        """Get the form class for group object permissions management.
+        Parameters:
+            request (HttpRequest): The HTTP request object.
+
+        Returns:
+            The form class for group object permissions management.
+                Default is `GroupManage`
         """
         return GroupManage
 
-    def get_obj_perms_manage_user_form(self, request):
-        """
-        Returns form class for user object permissions management.  By default
-        :form:`AdminUserObjectPermissionsForm` is returned.
+    def get_obj_perms_manage_user_form(self, request: HttpRequest) -> Type[forms.Form]:
+        """Get the form class for user object permissions management.
+
+        Parameters:
+            request (HttpRequest): The HTTP request object.
+
+        Returns:
+            The form class for user object permissions management.
+                Default is `AdminUserObjectPermissionsForm`.
         """
         return AdminUserObjectPermissionsForm
 
     def obj_perms_manage_group_view(self, request, object_pk, group_id):
-        """
-        Manages selected groups' permissions for current object.
-        """
+        """Manages selected groups' permissions for the current object."""
         if not self.has_change_permission(request, None):
             post_url = reverse('admin:index', current_app=self.admin_site.name)
             return redirect(post_url)
 
-        group = get_object_or_404(Group, id=group_id)
         obj = get_object_or_404(self.get_queryset(request), pk=object_pk)
+        GroupModel = get_group_obj_perms_model(obj).group.field.related_model
+        group = get_object_or_404(GroupModel, id=group_id)
         form_class = self.get_obj_perms_manage_group_form(request)
         form = form_class(group, obj, request.POST or None)
 
         if request.method == 'POST' and form.is_valid():
             form.save_obj_perms()
-            msg = ugettext("Permissions saved.")
+            msg = gettext("Permissions saved.")
             messages.success(request, msg)
             info = (
                 self.admin_site.name,
@@ -329,94 +349,72 @@ class GuardedModelAdminMixin:
         return render(request, self.get_obj_perms_manage_group_template(), context)
 
     def get_obj_perms_manage_group_template(self):
-        """
-        Returns object permissions for group admin template.  May be overridden
-        if need to change it dynamically.
+        """Returns object permissions for group admin template.
 
-        .. note::
-           If ``INSTALLED_APPS`` contains ``grappelli`` this function would
-           return ``"admin/guardian/grappelli/obj_perms_manage_group.html"``.
+        May be overridden if dynamic behavior is needed.
 
+        Returns:
+            template name
+
+        Note:
+           If `INSTALLED_APPS` contains `grappelli` this function would
+           return `"admin/guardian/grappelli/obj_perms_manage_group.html"`.
         """
         if 'grappelli' in settings.INSTALLED_APPS:
             return 'admin/guardian/contrib/grappelli/obj_perms_manage_group.html'
         return self.obj_perms_manage_group_template
 
     def get_obj_perms_manage_group_form(self, request):
-        """
-        Returns form class for group object permissions management.  By default
-        :form:`AdminGroupObjectPermissionsForm` is returned.
+        """Get the form class for group object permissions management.
+
+        Parameters:
+            request (HttpRequest): The HTTP request object.
+
+        Returns:
+            The form class for group object permissions management.
+                Default is `AdminGroupObjectPermissionsForm`.
         """
         return AdminGroupObjectPermissionsForm
 
 
 class GuardedModelAdmin(GuardedModelAdminMixin, admin.ModelAdmin):
-    """
-    Extends ``django.contrib.admin.ModelAdmin`` class. Provides some extra
-    views for object permissions management at admin panel. It also changes
-    default ``change_form_template`` option to
-    ``'admin/guardian/model/change_form.html'`` which is required for proper
+    """Provide views for managing object permissions on the Django admin panel.
+
+    Extends the `django.contrib.admin.ModelAdmin` class.
+    It uses `'admin/guardian/model/change_form.html'` as the
+    default `change_form_template` attribute which is required for proper
     url (object permissions related) being shown at the model pages.
 
-    **Extra options**
+    Attributes:
+        obj_perms_manage_template (str): Defaults to `admin/guardian/model/obj_perms_manage.html`
+        obj_perms_manage_user_template (str): Defaults to `admin/guardian/model/obj_perms_manage_user.html`
+        obj_perms_manage_group_template (str): Defaults to `admin/guardian/model/obj_perms_manage_group.html`
+        user_can_access_owned_objects_only (bool): Defaults to `False`.
+            If `True`, `request.user` would be used to filter out objects they don't own
+            checking `user` field of used model -
+            field name may be overridden by the `user_owned_objects_field` option.
+        user_can_access_owned_by_group_objects_only (bool): *Default*: `False`
+            If `True`, `request.user` is used to filter out objects her or his group doesn't own
+            (checking if any group the user belongs to is set as `group` field of the object;
+            name of the field can be changed by overriding `group_owned_objects_field`).
+        group_owned_objects_field (str): *Default*: `group`
+            Name of the field to check for group ownership.
+        include_object_permissions_urls (bool): *Default*: `True`
+            Added in version 1.2.
+            If `False` guardian-specific URLs are **NOT** included in the admin
 
-    ``GuardedModelAdmin.obj_perms_manage_template``
+    Warning:
+       Setting `user_can_access_owned_objects_only` to `True` will **NOT** affect superusers!
+       Admins would still see all items.
 
-        *Default*: ``admin/guardian/model/obj_perms_manage.html``
+    Warning:
+       Setting `user_can_access_owned_by_group_objects_only` to `True` will **NOT** affect superusers!
+       Admins would still see all items.
 
-    ``GuardedModelAdmin.obj_perms_manage_user_template``
 
-        *Default*: ``admin/guardian/model/obj_perms_manage_user.html``
-
-    ``GuardedModelAdmin.obj_perms_manage_group_template``
-
-        *Default*: ``admin/guardian/model/obj_perms_manage_group.html``
-
-    ``GuardedModelAdmin.user_can_access_owned_objects_only``
-
-        *Default*: ``False``
-
-        If this would be set to ``True``, ``request.user`` would be used to
-        filter out objects he or she doesn't own (checking ``user`` field
-        of used model - field name may be overridden by
-        ``user_owned_objects_field`` option).
-
-        .. note::
-           Please remember that this will **NOT** affect superusers!
-           Admins would still see all items.
-
-    ``GuardedModelAdmin.user_can_access_owned_by_group_objects_only``
-
-        *Default*: ``False``
-
-        If this would be set to ``True``, ``request.user`` would be used to
-        filter out objects her or his group doesn't own (checking if any group
-        user belongs to is set as ``group`` field of the object; name of the
-        field can be changed by overriding ``group_owned_objects_field``).
-
-        .. note::
-           Please remember that this will **NOT** affect superusers!
-           Admins would still see all items.
-
-    ``GuardedModelAdmin.group_owned_objects_field``
-
-        *Default*: ``group``
-
-    ``GuardedModelAdmin.include_object_permissions_urls``
-
-        *Default*: ``True``
-
-        .. versionadded:: 1.2
-
-        Might be set to ``False`` in order **NOT** to include guardian-specific
-        urls.
-
-    **Usage example**
-
-    Just use :admin:`GuardedModelAdmin` instead of
-    ``django.contrib.admin.ModelAdmin``.
-
-    .. code-block:: python
+    Example:
+        ```python
+        # Just use GuardedModelAdmin instead of django.contrib.admin.ModelAdmin.
 
         from django.contrib import admin
         from guardian.admin import GuardedModelAdmin
@@ -426,7 +424,7 @@ class GuardedModelAdmin(GuardedModelAdminMixin, admin.ModelAdmin):
             pass
 
         admin.site.register(Author, AuthorAdmin)
-
+        ```
     """
 
 
@@ -440,9 +438,7 @@ class UserManage(forms.Form):
                            )
 
     def clean_user(self):
-        """
-        Returns ``User`` instance based on the given identification.
-        """
+        """Returns `User` instance based on the given identification."""
         identification = self.cleaned_data['user']
         user_model = get_user_model()
         try:
@@ -462,13 +458,12 @@ class GroupManage(forms.Form):
                                                            _("This group does not exist")})
 
     def clean_group(self):
-        """
-        Returns ``Group`` instance based on the given group name.
-        """
+        """Returns `Group` instance based on the given group name."""
         name = self.cleaned_data['group']
+        GroupModel = get_group_obj_perms_model().group.field.related_model
         try:
-            group = Group.objects.get(name=name)
+            group = GroupModel.objects.get(name=name)
             return group
-        except Group.DoesNotExist:
+        except GroupModel.DoesNotExist:
             raise forms.ValidationError(
                 self.fields['group'].error_messages['does_not_exist'])
